@@ -1,4 +1,4 @@
-# Copyright (C) 1998,1999,2000 by the Free Software Foundation, Inc.
+# Copyright (C) 1998,1999,2000,2001 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -186,7 +186,7 @@ def PrintPostRequest(mlist, id, info, total, count, form):
     t.AddRow([Bold('From:'), sender])
     row, col = t.GetCurrentRowIndex(), t.GetCurrentCellIndex()
     t.AddCellInfo(row, col-1, align='right')
-    t.AddRow([Bold('Subject:'), subject])
+    t.AddRow([Bold('Subject:'), cgi.escape(subject)])
     t.AddCellInfo(row+1, col-1, align='right')
     t.AddRow([Bold('Reason:'), reason])
     t.AddCellInfo(row+2, col-1, align='right')
