@@ -1,4 +1,4 @@
-# Copyright (C) 1998,1999,2000 by the Free Software Foundation, Inc.
+# Copyright (C) 1998,1999,2000,2001 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -29,8 +29,6 @@ import string
 import re
 from UserDict import UserDict
 from types import StringType
-# XXX: obsolete, should use re module
-import regsub
 import random
 import urlparse
 
@@ -415,19 +413,8 @@ def CheckSiteAdminPassword(pw1):
 
 
 def QuoteHyperChars(str):
-    arr = regsub.splitx(str, '[<>"&]')
-    i = 1
-    while i < len(arr):
-	if arr[i] == '<':
-	    arr[i] = '&lt;'
-	elif arr[i] == '>':
-	    arr[i] = '&gt;'
-	elif arr[i] == '"':
-	    arr[i] = '&quot;'
-	else:     #if arr[i] == '&':
-	    arr[i] = '&amp;'
-	i = i + 2
-    return string.join(arr, '')
+    from cgi import escape
+    return escape(str, quote=1)
 
 
 
